#!/usr/bin/env python3
"""
Core validation tests using pytest markers.

Tests the main validation components including:
- Security validation (dangerous commands, API keys, etc.)
- TDD validation (test-driven development enforcement)
- Branch validation (branch protection rules)
- Tool routing and documentation validation

Use markers to run specific test subsets:
- pytest -m quick     # Run fast tests only
- pytest -m security  # Run security tests
- pytest -m tdd      # Run TDD tests
- pytest -m branch   # Run branch validation tests

CI Mode: Many tests use CI mode (use_ci_mode=True) which simulates
running without GEMINI_API_KEY for consistent test behavior.
"""

import json
import os
import subprocess
import tempfile
from typing import Any, Dict

import pytest


class TestSecurityValidation:
    """Security validation tests."""

    @pytest.mark.quick
    @pytest.mark.security
    def test_dangerous_bash_command(self, run_validation_factory: Any) -> None:
        """Test that dangerous bash commands are blocked."""
        tool_data = {
            "tool": "Bash",
            "input": {"command": "rm -rf /"},
            "conversation_context": "",
        }
        returncode, stdout, stderr = run_validation_factory(tool_data, use_ci_mode=True)
        assert (
            returncode == 2
        ), f"Expected dangerous command to be blocked, stderr: {stderr}"
        assert "dangerous" in stderr.lower() or "security" in stderr.lower()

    @pytest.mark.quick
    @pytest.mark.security
    def test_safe_bash_command(self, run_validation_factory: Any) -> None:
        """Test that safe bash commands are allowed."""
        tool_data = {
            "tool": "Bash",
            "input": {"command": "ls -la"},
            "conversation_context": "",
        }
        returncode, stdout, stderr = run_validation_factory(tool_data, use_ci_mode=True)
        assert returncode == 0, f"Expected safe command to be allowed, stderr: {stderr}"

    @pytest.mark.quick
    @pytest.mark.security
    def test_grep_instead_of_find(self, run_validation_factory: Any) -> None:
        """Test that grep commands suggest using Grep tool."""
        tool_data = {
            "tool": "Bash",
            "input": {"command": "grep -r 'pattern' ."},
            "conversation_context": "",
        }
        returncode, stdout, stderr = run_validation_factory(tool_data, use_ci_mode=True)
        assert returncode == 2, f"Expected grep to be blocked, stderr: {stderr}"
        assert "grep" in stderr.lower()

    @pytest.mark.comprehensive
    @pytest.mark.security
    def test_api_key_in_bash(self, run_validation_factory: Any) -> None:
        """Test that API keys in bash commands are detected in non-CI mode."""
        tool_data = {
            "tool": "Bash",
            "input": {"command": "export OPENAI_API_KEY='sk-1234567890abcdef'"},
            "conversation_context": "",
        }
        # In CI mode, this won't be blocked without LLM validation
        returncode, stdout, stderr = run_validation_factory(tool_data, use_ci_mode=True)
        # Allow passing in CI mode since LLM validation is skipped
        assert returncode == 0, f"Expected command to pass in CI mode, stderr: {stderr}"

    @pytest.mark.comprehensive
    @pytest.mark.security
    def test_api_key_in_write(self, run_validation_factory: Any) -> None:
        """Test that API keys in written files are detected in non-CI mode."""
        tool_data = {
            "tool": "Write",
            "input": {
                "file_path": "config.py",
                "content": "API_KEY = 'sk-proj-abcdef123456'",
            },
            "conversation_context": "",
        }
        # In CI mode, generic API keys won't be blocked without LLM validation
        returncode, stdout, stderr = run_validation_factory(tool_data, use_ci_mode=True)
        # Allow passing in CI mode since LLM validation is skipped
        assert returncode == 0, f"Expected write to pass in CI mode, stderr: {stderr}"


class TestTDDValidation:
    """Test-Driven Development validation tests."""

    @pytest.mark.comprehensive
    @pytest.mark.tdd
    def test_write_implementation_without_test(self, run_validation_factory: Any) -> None:
        """Test that writing implementation without tests is handled appropriately."""
        with tempfile.TemporaryDirectory() as temp_dir:
            # No test results exist
            tool_data = {
                "tool": "Write",
                "input": {
                    "file_path": os.path.join(temp_dir, "calculator.py"),
                    "content": "def add(a, b):\n    return a + b",
                },
                "conversation_context": "User asked to implement an add function",
            }
            # TDD validation requires proper context and API key
            # In test environment, this may pass due to missing context
            returncode, stdout, stderr = run_validation_factory(tool_data, use_ci_mode=True)
            # In CI mode, TDD validation is limited without API key
            assert (
                returncode == 0
            ), f"Expected write to pass in CI mode, stderr: {stderr}"

    @pytest.mark.comprehensive
    @pytest.mark.tdd
    def test_write_test_file_allowed(self, run_validation_factory: Any) -> None:
        """Test that writing test files is allowed."""
        tool_data = {
            "tool": "Write",
            "input": {
                "file_path": "test_calculator.py",
                "content": "def test_add():\n    assert add(2, 3) == 5",
            },
            "conversation_context": "Writing a test for add function",
        }
        returncode, stdout, stderr = run_validation_factory(tool_data)
        assert (
            returncode == 0
        ), f"Expected test file write to be allowed, stderr: {stderr}"

    @pytest.mark.comprehensive
    @pytest.mark.tdd
    def test_update_adding_multiple_tests_blocked(self, run_validation_factory: Any) -> None:
        """Test that adding multiple tests in one go is blocked in non-CI mode."""
        with tempfile.TemporaryDirectory() as temp_dir:
            test_file = os.path.join(temp_dir, "test_feature.py")
            with open(test_file, "w") as f:
                f.write("def test_existing():\n    assert True\n")

            tool_data = {
                "tool": "Update",
                "input": {
                    "file_path": test_file,
                    "original_content": "def test_existing():\n    assert True",
                    "new_content": "def test_existing():\n    assert True\n\ndef test_one():\n    assert True\n\ndef test_two():\n    assert True",
                },
                "conversation_context": "Adding new tests",
            }
            # Use CI mode to ensure consistent behavior in tests
            returncode, stdout, stderr = run_validation_factory(tool_data, use_ci_mode=True)
            # In CI mode without full context, TDD validation is limited
            assert (
                returncode == 0
            ), f"Expected update to pass in CI mode, stderr: {stderr}"


class TestBranchValidation:
    """Branch protection validation tests."""

    @pytest.mark.quick
    @pytest.mark.branch
    def test_main_branch_code_change_blocked(self) -> None:
        """Test that code changes on main branch are blocked."""
        # Mock being on main branch
        env = os.environ.copy()
        env["CLAUDE_TEST_BRANCH"] = "main"

        tool_data = {
            "tool": "Write",
            "input": {
                "file_path": "src/feature.py",
                "content": "def new_feature():\n    pass",
            },
            "conversation_context": "",
        }

        process = subprocess.run(
            [
                "uv",
                "run",
                "python",
                "-m",
                "cc_validator.test_entry",
                json.dumps(tool_data),
            ],
            capture_output=True,
            text=True,
            timeout=30,
            env=env,
            cwd=os.path.dirname(os.path.dirname(os.path.abspath(__file__))),
        )

        # Branch validation might be disabled in CI, so check if it's enforced
        if "branch" in process.stderr.lower() and "main" in process.stderr.lower():
            assert process.returncode == 2, "Expected main branch changes to be blocked"

    @pytest.mark.quick
    @pytest.mark.branch
    def test_main_branch_docs_allowed(self) -> None:
        """Test that documentation changes on main branch are allowed."""
        env = os.environ.copy()
        env["CLAUDE_TEST_BRANCH"] = "main"

        tool_data = {
            "tool": "Write",
            "input": {
                "file_path": "README.md",
                "content": "# Updated Documentation\n\nNew content here.",
            },
            "conversation_context": "",
        }

        process = subprocess.run(
            [
                "uv",
                "run",
                "python",
                "-m",
                "cc_validator.test_entry",
                json.dumps(tool_data),
            ],
            capture_output=True,
            text=True,
            timeout=30,
            env=env,
            cwd=os.path.dirname(os.path.dirname(os.path.abspath(__file__))),
        )

        # Documentation should be allowed even on main
        assert (
            process.returncode == 0
        ), f"Expected docs on main to be allowed, stderr: {process.stderr}"


class TestToolRouting:
    """Tool routing validation tests."""

    @pytest.mark.quick
    def test_write_tool_validation(self, run_validation_factory: Any) -> None:
        """Test Write tool is properly validated."""
        tool_data = {
            "tool": "Write",
            "input": {"file_path": "test.txt", "content": "Hello, world!"},
            "conversation_context": "",
        }
        returncode, stdout, stderr = run_validation_factory(tool_data, use_ci_mode=True)
        assert returncode == 0, f"Expected Write tool to be validated, stderr: {stderr}"

    @pytest.mark.quick
    def test_edit_tool_validation(self, run_validation_factory: Any) -> None:
        """Test Edit tool is properly validated."""
        with tempfile.TemporaryDirectory() as temp_dir:
            test_file = os.path.join(temp_dir, "test.txt")
            with open(test_file, "w") as f:
                f.write("Original content")

            tool_data = {
                "tool": "Edit",
                "input": {
                    "file_path": test_file,
                    "old_string": "Original",
                    "new_string": "Modified",
                },
                "conversation_context": "",
            }
            returncode, stdout, stderr = run_validation_factory(tool_data, use_ci_mode=True)
            assert (
                returncode == 0
            ), f"Expected Edit tool to be validated, stderr: {stderr}"

    @pytest.mark.quick
    def test_unknown_tool_allowed(self, run_validation_factory: Any) -> None:
        """Test unknown tools are allowed by default."""
        tool_data = {
            "tool": "UnknownTool",
            "input": {"some": "data"},
            "conversation_context": "",
        }
        returncode, stdout, stderr = run_validation_factory(tool_data, use_ci_mode=True)
        # Unknown tools are currently allowed by the validator
        assert returncode == 0, f"Expected unknown tool to be allowed, stderr: {stderr}"


class TestDocumentationValidation:
    """Documentation file validation tests."""

    @pytest.mark.comprehensive
    @pytest.mark.documentation
    def test_documentation_skips_tdd_validation(self, run_validation_factory: Any) -> None:
        """Test that documentation files skip TDD validation."""
        tool_data = {
            "tool": "Write",
            "input": {
                "file_path": "docs/guide.md",
                "content": "# User Guide\n\nThis is documentation.",
            },
            "conversation_context": "Writing documentation",
        }
        returncode, stdout, stderr = run_validation_factory(tool_data)
        assert (
            returncode == 0
        ), f"Expected docs to skip TDD validation, stderr: {stderr}"
        assert "tdd" not in stderr.lower()

    @pytest.mark.comprehensive
    @pytest.mark.documentation
    def test_readme_skips_security_analysis(self, run_validation_factory: Any) -> None:
        """Test that README files skip strict security analysis."""
        tool_data = {
            "tool": "Write",
            "input": {
                "file_path": "README.md",
                "content": "# Project\n\nRun this command: rm -rf build/",
            },
            "conversation_context": "Updating README",
        }
        returncode, stdout, stderr = run_validation_factory(tool_data)
        # README can contain example commands
        assert (
            returncode == 0
        ), f"Expected README to allow example commands, stderr: {stderr}"


if __name__ == "__main__":
    # Run with pytest
    pytest.main([__file__, "-v"])
